require 'rho/rhocontroller'
require 'helpers/browser_helper'
require 'date'

class FriendController < Rho::RhoController
  include BrowserHelper
  
  # Display all stored friend activity and categorize them by date.
  def index
    @all_friends = []
    @today_friends_activity = []
    @older_friends_activity = []
    
    @all_friends = Friend.find(:all)
    if @all_friends.any?    
      friends = @all_friends.uniq.sort_by{|friend| friend.created_at}.reverse
      @today_friends_activity = Friend.get_today_friends_array(friends)
      @older_friends_activity = Friend.get_older_friends_array(friends)
    end
    render
  end
  
  # This action gets called by the tabbar and checks if there are available services.
  def update_friends
    if users.empty?
      NativeBar.switch_tab(2)
      WebView.navigate(url_for(:controller => :User, :action => 'new'), 2)
    else
      if User.reload_friends? || Friend.empty?
        Friend.delete_all
        get_friends_activity
      else
        WebView.navigate(url_for(:action => :index), 1)
      end
    end
  end
  
  # This action is called by the Refresh button in Friends index. It forces the page to reload it 
  def update_friends_from_link
    Friend.delete_all
    get_friends_activity
  end
  
  # Gets called once for every service
  def get_friends_activity
    @users = services_not_checked_for_friends
    if @users.empty?
      User.unchecked_friends
      WebView.navigate(url_for(:action => :index), 1)
    else
      @user = @users.first
      @user.friends = "true"
      @user.save
      Rho::AsyncHttp.get(
        :url => friends_url(@user), # Abstract the API url into a helper method
        :callback => (url_for :action => :activity_callback),
        :headers => {'User-Agent' => Rho::RhoConfig.user_agent},
        :callback_param => "user_id=#{@user.object}",
        :authentication => {:type => :basic, :username => @user.login, :password => @user.password} )
    end
  end
  
  # Receives the callback from the API and stores the friends activity locally. When we are done fetching activity from 
  # all services it redirects to the index
  def activity_callback
    @user = User.find(@params['user_id'])
    friends = Friend.get_friends_hash(@params['body'], @user.service)
    Friend.save_friends(friends)
    @user.last_check_friends = Time.now.strftime("%I:%M")
    @user.save
    
    if remaining_services_for_friends_activity?
      WebView.execute_js("$.get('/app/Friend/get_friends_activity');",1)
    else
      User.unchecked_friends
      WebView.navigate(url_for(:action => :index), 1)
    end  
  end
end
