# The model has already been created by the framework, and extends Rhom::RhomObject
# You can add more methods here
require 'date'

class Place
  include Rhom::PropertyBag
  
  def from_gowalla?
    self.gw_id && !self.gw_id.empty?
  end
  
  def from_foursquare?
    self.fs_id && !self.fs_id.empty?
  end
  
  def self.empty?
    return find(:all).empty?
  end
  
  def self.build_from_venue(venue)
    return new(Place.attributes_hash_from_venue(venue))
  end
  
  def self.build_from_spot(spot)
    return new(Place.attributes_hash_from_spot(spot))
  end
  
  def self.build_from_hash(attributes_hash)
    return new(attributes_hash)
  end

  def self.attributes_hash_from_venue(venue)
    fs_id = venue['id']
    name = venue['name']
    category = venue['primarycategory'] ? venue['primarycategory']['nodename'] : ""
    fs_image = venue['primarycategory'] ? venue['primarycategory']['iconurl'] : "/public/images/icon29x29.png"
    address = venue['address'] || ""
    city = venue['city'] || ""
    state = venue['state'] || ""
    zip = venue['zip'] || ""
    lat = venue['geolat']
    long = venue['geolong']
    attributes_hash = {:fs_id => fs_id, :name => name, :category => category, :image => fs_image, :address => address, 
                        :city => city, :state => state, :zip => zip, :lat => lat, :long => long}
    return attributes_hash
  end
    
  def self.attributes_hash_from_spot(spot)
    gw_id = spot["url"].gsub("/spots/", "")
    name = spot['name']
    category = spot['spot_categories'][0]['name'] ? spot['spot_categories'][0]['name'] : ""
    gw_image = spot['image_url']
    address = ""
    city = spot['address']['locality'] || ""
    state = spot['address']['region'] || ""
    zip = ""
    lat = spot['lat']
    long = spot['lng']
    attributes_hash = {:gw_id => gw_id, :name => name, :category => category, :image => gw_image, :address => address, 
                        :city => city, :state => state, :zip => zip, :lat => lat, :long => long}
    return attributes_hash
  end
  
  def self.attributes_hash_from_place(place)
    bk_id = place['id']
    name = place['name']
    category = ""
    bk_image = "/public/images/icon.png"
    address = place['street'] || ""
    city = place['city'] || ""
    state = place['state'] || ""
    zip = place['zip'] || ""
    lat = place['latitude']
    long = place['longitude']
    attributes_hash = {:bk_id => bk_id, :name => name, :category => category, :image => bk_image, :address => address, :city => city, :state => state, :zip => zip, :lat => lat, :long => long}
    return attributes_hash
  end
  
  def self.build_annotarions_array
    places = find(:all)
    annotations = Array.new
    places.each do |place|
      annotations << {:latitude => place.lat, :longitude => place.long, :title => place.name, :subtitle => place.category}
    end
    return annotations
  end
  
  def self.attributes_hash_from_fs_checkin(checkin)
    {:date => Date.parse(checkin["created"]).strftime('%d/%m'), :user => checkin["user"]["firstname"] + " " + checkin["user"]["lastname"], :photo => checkin["user"]["photo"]}
  end
  
  def self.attributes_hash_from_gw_checkin(checkin)
    {:date => Date.parse(checkin["created_at"]).strftime('%d/%m'), :user => checkin["user"]["first_name"] + " " + checkin["user"]["last_name"], :photo => checkin["user"]["image_url"]}
  end
  
  def self.attributes_hash_from_bk_checkin(checkin)
    {:date => Date.parse(checkin['created_at']).strftime('%d/%m'), :user => checkin['creator']['login'], :photo => checkin['creator']['smaller_avatar_url']}
  end
  
  def self.merge_current_and_new_places(places_hash)
    current_places = find(:all)
    new_places = []
    current_places.each do |place|
      place_hash_found = places_hash.detect {|place_hash| place_hash[:name].downcase.gsub("'", "") == place.name.downcase.gsub("'", "")}
      if place_hash_found
        if place_hash_found.has_key?(:fs_id)
          place.fs_id = place_hash_found[:fs_id]
          place.category = place_hash_found[:category] if place.category.empty?
          place.image = place_hash_found[:image] if place.image.empty?
        elsif place_hash_found.has_key?(:gw_id)
          place.gw_id = place_hash_found[:gw_id]
          place.image = place_hash_found[:image]
          place.category = place_hash_found[:category] if place.category.empty?
        elsif place_hash_found.has_key?(:bk_id)
          place.bk_id = place_hash_found[:bk_id]
        end
        place.save
        places_hash.delete(place_hash_found)
      end
    end
    
    places_hash.each do |place_hash|
      new_place = new(place_hash)
      new_place.save
    end
  end
  
  def self.get_places_hash(params, api_service)
    places = []
    if api_service == 'foursquare'
      return places if params.nil? || params['groups'].nil?
        params['groups'][-1]['venues'].each do |venue|
        places << Place.attributes_hash_from_venue(venue)
      end
    elsif api_service == 'gowalla'
      return places if params['spots'].empty?
      params['spots'][0..9].each do |spot|
        places << Place.attributes_hash_from_spot(spot) 
      end
    else
      return places if params.empty?
      params.each do |place|
        places << Place.attributes_hash_from_place(place)
      end
    end
    
    return places
  end
  
end
