require 'rho/rhocontroller'
require 'helpers/browser_helper'
require 'date'

class PlaceController < Rho::RhoController
  include BrowserHelper
  
  # Renders all stored places
  def index
    @places = Place.find(:all)
    @location = Location.find(:first)
    render
  end
  
  # First action called, it will check if there are any services registered (Foursquare/Gowalla/Brightkite)
  def show_places
    if users.empty?
      NativeBar.switch_tab(2)
      WebView.navigate(url_for(:controller => :User, :action => 'new'), 2)
    else
      get_places
    end
  end
  
  # Determines if we already have places stored and if we have a location. Also handles the case where we retrieved the location
  # based on the zipcode entered by the user.
  def get_places
    @places = Place.find(:all)
    @location = Location.find(:first)

    if @location && @places.any?
      if GeoLocation.known_position?
        update_location
      elsif !@location.zipcode?
        get_user_location
      else
        WebView.navigate(url_for(:action => 'index'), 0)
      end
    else
      get_nearby_places
    end
  end
  
  # Displays all places on a Native Mapping View.
  def showmap
    @location = Location.find(:first)
    @annotations = Place.build_annotarions_array
    
    map_params = {:settings => {:map_type => "standard", :region => [@location.lat, @location.long, 0.02, 0.02],
                                :zoom_enabled => true, :scroll_enabled => true, :shows_user_location => false,
                                :api_key => 'ABQIAAAAwLGh2EC-H_isIE-lAKj-bBSHKapxWSPci0U3ai7oYVg4d8GX_hTPEB-G16OhWWHZCTvA4xkqc6iRlA'},
                  :annotations => @annotations}
                  
    MapView.create map_params
  end
  
  # This action gets called once for every service available. It uses a Rho:AsyncHttp call to access the third party API's
  def get_nearby_places
    
    @location = Location.find(:first)
    if @location
      @users = services_not_checked_for_places
      if @users.empty?
        User.uncheck_all
        WebView.navigate(url_for(:action => 'index'), 0)
      else
        @user = @users.first
        Rho::AsyncHttp.get(
          :url => places_url(@user, @location.lat, @location.long), # Helper method to abrasct the url for the desired service.
          :callback => (url_for :action => :nearby_places_callback),
          :headers => {'User-Agent' => Rho::RhoConfig.user_agent},
          :callback_param => "service=#{@user.service}",
          :authentication => {:type => :basic, :username => @user.login, :password => @user.password} )
        
        # The current service gets flaged so the next time this method gets called it doesn't check the same service again.
        @user.checked = "true"
        @user.save
      end
    else
      get_user_location
    end
  end
  
  # Recieves the information from the API, parses the data and stores the places in the local database. It tries to merge
  # the places based on the name
  def nearby_places_callback
    places_hash = Place.get_places_hash(@params['body'], @params['service'])
    if Place.empty?
      places_hash.each do |place_hash|
        place = Place.build_from_hash(place_hash)
        place.save
      end
    else
      Place.merge_current_and_new_places(places_hash)
    end
    
    # If we are done checking all services, we navigate to the index to display all places. Also the services are unflaged.
    # Otherwise the get_nearby_places gets called for the next service.
    if services_not_checked_for_places.empty?
      User.uncheck_all
      WebView.navigate(url_for(:action => 'index'), 0)
    else
      get_nearby_places
    end
  end
  
  # This action gets called in the background when places are already stored. It updates the user location, and in case it is a
  # different one, it deletes all places and starts fetching all places again.
  def update_location
    @location = Location.find(:first)
    if (@params['known_position'].to_i != 0 && @params['status'] =='ok') || GeoLocation.known_position?
      if @location.different_location?
        Place.delete_all
        @location.lat = GeoLocation.latitude
        @location.long = GeoLocation.longitude
        @location.zipcode = ""
        @location.save
        get_nearby_places
      else
        WebView.navigate(url_for(:action => 'index'), 0)
      end
    end
  end
  
  # Retrieve the current user location and store it in the device.
  def get_user_location
    Place.delete_all
    Location.delete_all
    if (@params['known_position'].to_i != 0 && @params['status'] =='ok') || GeoLocation.known_position?
      @location = Location.new({:lat => GeoLocation.latitude, :long => GeoLocation.longitude, :zipcode => ""})
      @location.save
      WebView.navigate(url_for(:action => :show_places))
    else
      GeoLocation.set_notification(url_for(:action => :geo_callback), "", 10)
    end
  end
  
  # The callback for the notification when the GeoLocation is locked.
  def geo_callback
    get_user_location if @params['known_position'].to_i != 0 && @params['status'] =='ok'
    WebView.navigate(url_for(:controller => :Location, :action => 'new'), 0) if @params['available'].to_i == 0 || @params['status'] !='ok'
  end
  
  # Displays detailed information for the current place.
  def show
    @places = Place.find(:all)
    @place = Place.find(@params['id'])
    if @place
      render :action => :show
    else
      redirect :action => :index
    end
  end
  
  # Shows the map for a specific place
  def place_map
    @place = Place.find(@params['id'])
    map_params = {:settings => {:map_type => "standard", :region => [@place.lat, @place.long, 0.01, 0.01],
                                :zoom_enabled => true, :scroll_enabled => true, :shows_user_location => false,
                                :api_key => 'ABQIAAAAwLGh2EC-H_isIE-lAKj-bBSHKapxWSPci0U3ai7oYVg4d8GX_hTPEB-G16OhWWHZCTvA4xkqc6iRlA'},
                  :annotations => [{:latitude => @place.lat, 
                                    :longitude => @place.long, 
                                    :title => @place.name, 
                                    :subtitle => @place.category}]}
    MapView.create map_params
  end
  
  def get_checkins
    @place = Place.find(@params['id'])
    @users = services_not_checked_for_checkins 
    if @users.empty?
      User.mark_as_checked_checkin
      WebView.execute_js("if ($('.checkins ul.#{@place.object} li').size() == 1) {$('.checkins ul.#{@place.object} li.spinner span').show(); $('.checkins ul.#{@place.object} li.spinner img').hide();} else {$('.checkins ul.#{@place.object} li.spinner').hide();};", 0)
    else
      @user = services_not_checked_for_checkins.first      
      @user.check_ins = "true"
      @user.save
      
      if place_id_for_service(@user.service, @place)
        Rho::AsyncHttp.get(
          :url => checkins_url(@user, @place),
          :callback => (url_for :action => :get_checkins_callback),
          :headers => {'User-Agent' => Rho::RhoConfig.user_agent},
          :callback_param => "id=#{@place.object}&user_id=#{@user.object}",
          :authentication => {:type => :basic, :username => @user.login, :password => @user.password} )
      else
        if remaining_services?
          WebView.execute_js("$.get('/app/Place/get_checkins?id={#{@place.object}}')", 0)
        else
          User.mark_as_checked_checkin
          WebView.execute_js("if ($('.checkins ul.#{@place.object} li').size() == 1) {$('.checkins ul.#{@place.object} li.spinner span').show(); $('.checkins ul.#{@place.object} li.spinner img').hide();} else {$('.checkins ul.#{@place.object} li.spinner').hide();};", 0)
        end
      end
    end
  end
  
  def get_checkins_callback
    @place = Place.find(@params['id'])
    @user = User.find(@params['user_id'])
    checkins_hash = []    
    checkins_hash = get_checkins_hash(@params['body'], @user.service)
    
    friends_checkins = ""
    checkins_hash.each do |checkin|
     friends_checkins += "<li class=\"check_in\"><img src=\"#{checkin[:photo]}\" height=\"40px\" width=\"40px\" /><div>" + " #{checkin[:user]} " + "</div> <div>" + " #{checkin[:date]} " + "</div></li>"
    end
    
    WebView.execute_js("$('.checkins ul.#{@place.object}').append('#{friends_checkins}');", 0)
    
    if remaining_services?
      WebView.execute_js("$.get('/app/Place/get_checkins?id={#{@place.object}}')", 0)
    else
      User.mark_as_checked_checkin
      WebView.execute_js("if ($('.checkins ul.#{@place.object} li').size() == 1) {$('.checkins ul.#{@place.object} li.spinner span').show(); $('.checkins ul.#{@place.object} li.spinner img').hide();} else {$('.checkins ul.#{@place.object} li.spinner').hide();};", 0)
    end
  end
  
  def new_check_in
    @place = Place.find(@params['id'])  
  end
  
  def check_in
    @place = Place.find(@params['id'])
    @users = services_not_checked_for_check_in
    if @users.empty?
      WebView.execute_js("$('.check_in_form.#{@place.object} .spinner').hide();  $('.check_in_form.#{@place.object} .check_in_message').addClass('message error').text('This service is enabled for foursquare and brightkite services').show(); $('.check_in_form.#{@place.object} .close').show(); ", 0)
    else
      @user = @users.first
      @user.checked_check_in = "true"
      @user.save
      
      if place_id_for_service(@user.service, @place)
        shout = @params["comment"].empty? ? "" : "#{Rho::RhoSupport.url_encode(@params["comment"])}"
        Rho::AsyncHttp.post(
          :url => check_in_url(@user, @place, shout),
          :callback => (url_for :action => :check_in_callback),
          :headers => {'User-Agent' => Rho::RhoConfig.user_agent},
          :callback_param => "id=#{@place.object}&user_id=#{@user.object}&comment=#{shout}",
          :authentication => {:type => :basic, :username => @user.login, :password => @user.password} )
      else
        if remaining_services_check_in?
          WebView.execute_js("$.get('/app/Place/check_in?id=#{@place.object}&comment=#{shout}')", 0)
        else
          User.unchecked_check_in
          WebView.execute_js("$('.check_in_form .spinner').hide(); if ($('.check_in_form.#{@place.object} .check_in_message').text()==\"\" ){$('.check_in_form.#{@place.object} .check_in_message').addClass('message error').text('This service is enabled for foursquare and brightkite services').show();}else{$('.check_in_form.#{@place.object} .check_in_message').addClass('message success').show();}; $('.check_in_form.#{@place.object} .close').show();", 0)
        end
      end
    end
  end
  
  def check_in_callback
    @place = Place.find(@params['id'])
    @user = User.find(@params['user_id'])
    shout = @params["comment"].empty? ? "" : "#{Rho::RhoSupport.url_encode(@params["shout"])}"
    WebView.execute_js("if ($('.check_in_form.#{@place.object} .check_in_message').text()== \"\"){$('.check_in_form.#{@place.object} .check_in_message').html('You have check-in in <span> #{escape_javascript(@place.name)} </span> with your #{@user.service} account');} else{$('.check_in_form.#{@place.object} .check_in_message').append(' and your #{@user.service} account');};", 0)
    if remaining_services_check_in?
      WebView.execute_js("$.get('/app/Place/check_in?id=#{@place.object}&comment=#{shout}')", 0)
    else
      User.unchecked_check_in
      WebView.execute_js("$('.check_in_form.#{@place.object} .spinner').hide(); $('.check_in_form.#{@place.object} .check_in_message').addClass('message success').show(); $('.check_in_form.#{@place.object} .close').show();", 0)
    end
  end
  
end

