require 'date'

module BrowserHelper

  def placeholder(label=nil)
    "placeholder='#{label}'" if (platform == 'apple' || platform == 'android')
  end

  def platform
    System::get_property('platform').downcase
  end

  def selected(option_value,object_value)
    "selected=\"yes\"" if option_value == object_value
  end

  def checked(option_value,object_value)
    "checked=\"yes\"" if option_value == object_value
  end
  
  def current_user
    @current_user ||= User.find(:first, :conditions => {:service => "gowalla"})
  end
  
  def users
    @current_users ||= User.find(:all)
  end
  
  def services_not_checked_for_places
    @users_no_places ||= User.find(:all, :conditions => {:checked => "false"})
  end
  
  def services_not_checked_for_checkins
    @users_no_checkins ||= User.find(:all, :conditions => {:check_ins => "false"})
  end
  
  def services_not_checked_for_check_in
    @users_no_check_in ||= User.find(:all, :conditions => {:checked_check_in => "false"})
  end
  
  def services_not_checked_for_friends
    @users_no_friends ||= User.find(:all, :conditions => {:friends => "false"})
  end
  
  def remaining_services?
    puts "AAAA: All users in remaining_services? #{User.find(:all).inspect} value #{@remaining_users.inspect} "
    @remaining_users ||= User.find(:all, :conditions => {:check_ins => "false"})
    puts "AAAA: User in remaining services? returning value #{!@remaining_users.empty?} value #{@remaining_users.inspect} "
    !@remaining_users.empty?
  end
  
  def remaining_services_check_in?
    puts "AAAA: All users in remaining_services_check_in? #{User.find(:all).inspect} value #{@remaining_users_check_in.inspect} "
    @remaining_users_check_in ||= User.find(:all, :conditions => {:checked_check_in => "false"})
    puts "AAAA: User in remaining services check-in? returning value #{!@remaining_users_check_in.empty?} value #{@remaining_users_check_in.inspect} "
    !@remaining_users_check_in.empty?
  end
  
  def remaining_services_for_friends_activity?
    @remaining_users_friends_activity ||= User.find(:all, :conditions => {:friends => "false"})
    puts "AAAA: User in remaining services friends_activity? returning value #{!@remaining_users_friends_activity.empty?} value #{@remaining_users_friends_activity.inspect} "
    !@remaining_users_friends_activity.empty?
  end
  
  def get_checkins_hash(params, service)
    checkins = []
    if service == "foursquare"
      puts "AAAA: Has checkins array? #{params["venue"]["checkins"].inspect}"
      if params["venue"]["checkins"]
        params["venue"]["checkins"].each do |checkin|
          checkins << Place.attributes_hash_from_fs_checkin(checkin)
        end
      end
    elsif service == "gowalla"
      if params['activity']
        params['activity'].each do |checkin|
          checkins << Place.attributes_hash_from_gw_checkin(checkin)
        end
      end
    else
      if params
        params.each do |checkin|
          checkins << Place.attributes_hash_from_bk_checkin(checkin)
        end
      end
    end
    puts "AAAA: Checkins hash: #{checkins.inspect}"
    return checkins
  end
  
  def check_in_message(params, service)
    puts "AAAA: params in check_in_message #{params.inspect}"
    message = ""
    if service == 'foursquare' and params
      message = params["checkin"]["message"]
      message = message.gsub(/@/, "<span>@").sub(/\./, ".</span>")
    elsif service == 'brightkite' and params
      puts "AAAA: creator #{params["creator"].inspect} creator-login #{params['creator']['login']} and place #{params["place"].inspect} and place-name #{params['place']['name']}"
      message = params["creator"]["login"] + " @ " + params["place"]["name"]
    end
    return escape_javascript(message)
  end
  
  def user_url(user)
    if user.service == 'foursquare'
      "http://api.foursquare.com/v1/user.json"
    elsif user.service == 'gowalla'
      "http://api.gowalla.com/users/#{user.login}.json"
    else
      "http://brightkite.com/people/#{user.login}.json"
    end
  end
  
  def places_url(user, lat, long, limit=10, radius=50)
    if user.service == 'foursquare'
      "http://api.foursquare.com/v1/venues.json?geolat=#{lat}&geolong=#{long}&l=#{limit}" 
    elsif user.service == 'gowalla'
      "http://api.gowalla.com/spots.json?lat=#{lat}&lng=#{long}&radius=#{radius}"
    elsif user.service == 'brightkite'
      "http://brightkite.com/places.json?clat=#{lat}&clng=#{long}&limit=#{limit}"
    else
      raise "Social Network service not allowed"
    end
  end
  
  def checkins_url(user, place)
    if user.service == "foursquare"
      "http://api.foursquare.com/v1/venue.json?vid=#{place.fs_id}"
    elsif user.service == "gowalla"
      "http://api.gowalla.com/spots/#{place.gw_id}/events.json"
    elsif user.service == "brightkite"
      "http://brightkite.com/places/#{place.bk_id}/objects.json?filters=checkins&since_ts=#{(Time.now-31536000).to_i}"
    else
      raise "Social Network service not allowed"
    end
  end
  
  def check_in_url(user, place, comment)
    if user.service == 'foursquare'
      "http://api.foursquare.com/v1/checkin.json?vid=#{place.fs_id}&shout=#{comment}"
    elsif user.service == 'brightkite'
      "http://brightkite.com/places/#{place.bk_id}/checkins.json"
    else
      raise "Social Network service not allowed"
    end
  end
  
  def friends_url(user)
    if user.service == 'foursquare'
      "http://api.foursquare.com/v1/checkins.json"
    elsif user.service == 'brightkite'
      "http://brightkite.com/people/#{user.login}/friends.json" 
    else
      raise "Social Network service not allowed"
    end
  end
  
  def place_id_for_service(service, place)
    place_id = nil
    if service == "foursquare"
      place_id = place.fs_id
    elsif service == "gowalla"
      place_id = place.gw_id
    elsif service == "brightkite"
      place_id = place.bk_id
    end
    return place_id
  end
  
  def fs_user
   @fs_user ||= User.find(:first, :conditions => {:service => "foursquare"})
 end
  
  def tabs
    [{ :label => "Places", :action => '/app/Place/show_places', :icon => "/public/images/marker.png", :reload => true}, 
             { :label => "Friends", :action => '/app/Friend/update_friends', :icon => "/public/images/friends.png", :reload => true},
              { :label => "Profile", :action => '/app/User/new', :icon => "/public/images/profile.png", :reload => false }]
  end
  
  def rounded_distance(lat1, long1, lat2, long2)
    distance = GeoLocation.haversine_distance(lat1.to_f, long1.to_f, lat2.to_f, long2.to_f)
    truncated = (distance*100).to_i
    return (truncated.to_f)/100
  end
  
  def truncate(text, size = 25)    
    (text.length > size ? text[0...size-1] + "..." : text).to_s if text
  end
  
  def title_width(place)
    if place.from_gowalla?
      "180px"
    elsif place.from_foursquare?
      "246px"
    else
      "220px"
    end
  end
  
  def places_html(places, location)
    puts "AAA: building the html for getting the places with location #{location.inspect}"
    places_html = ""
    
    if places.any?
      places.each do |place|
        gw_logo = place.gw_id && !place.gw_id.empty? ? "<img src='/public/images/gw_logo.png'/>" : ""
        fs_logo = place.fs_id && !place.fs_id.empty? ? "<img src='/public/images/fs_logo.png'/>" : ""
        bk_logo = place.bk_id && !place.bk_id.empty? ? "<img src='/public/images/bk_logo.png'/>" : ""
        places_html += "<li><a class='place_item' href='#' data-href='#{url_for :action => :show, :id => place.object}'><div class='info'><div class='title'>#{truncate(place.name, 20)}</div><div class='distance'>#{rounded_distance(place.lat, place.long, location.lat, location.long)} miles</div></div>"
        places_html += "<span class='logos'>#{gw_logo}#{fs_logo}#{bk_logo}</span></a></li>"
      end
    end
    
    return escape_javascript(places_html)
  end
  
  def build_html_for_friends_activity(friends, service)
    fs_logo = service == 'foursquare' ? "<img src='/public/images/fs_logo.png'/>" : ""
    bk_logo = service == 'brightkite' ? "<img src='/public/images/bk_logo.png'/>" : ""
    friends_list = "<ul>"
    friends.each do |friend|
      friends_list += "<li><img src=\"#{friend[:photo]}\" height=\"45px\" width=\"45px\" /> <div class=\"display\">" + "#{truncate(friend[:friend_at_place], 25)}" + "</div><div class=\"details\">#{fs_logo}#{bk_logo}<div class=\"created_at\">" + "#{friend[:created_at]}" + "</div><div class=\"comment\">" + "#{truncate(friend[:comment], 18)}" + "</div></div></li>"
    end
    friends_list += "</ul>"
    return escape_javascript(friends_list)
  end
  
  def display_if_has_zipcode
    @location ||= Location.find(:first)
    @location && @location.zipcode? ? "" : "display: none;"
  end
  
  def zipcode_if_available
    @location ||= Location.find(:first)
    @location && @location.zipcode? ? "#{@location.zipcode}" : ""
  end
  
  def display_if_places
    @places = Place.find(:all)
  end
  
  def gw_place?(place)
    gw_logo = place.gw_id && !place.gw_id.empty? ? "<img src='/public/images/gw_logo.png'/>" : ""
  end
  
  def fs_place?(place)
    fs_logo = place.fs_id && !place.fs_id.empty? ? "<img src='/public/images/fs_logo.png'/>" : ""
  end
  
  def bk_place?(place)
    bk_logo = place.bk_id && !place.bk_id.empty? ? "<img src='/public/images/bk_logo.png'/>" : ""
  end
  
  def friend_from?(service)
    logo_friend = ""
    if service == "foursquare"
      logo_friend = "<img src='/public/images/fs_logo.png'/>"
    elsif service == "brightkite"
      logo_friend = "<img src='/public/images/bk_logo.png'/>"
    end
    puts "AAAA: logo_friend #{logo_friend}"
    return logo_friend
  end
  
  def escape_javascript(javascript)
    js_escape_map = { '\\' => '\\\\', '</' => '<\/', "\r\n" => '\n', "\n" => '\n', "\r" => '\n', '"' => '\\"', "'" => "\\'" }
    if javascript
      javascript.gsub(/(\\|<\/|\r\n|[\n\r"'])/) { js_escape_map[$1] }
    else
      ''
    end
  end
     
end
